#include <errno.h>
#include <msp/strings/formatter.h>
#include "datagramsocket.h"

namespace Msp {
namespace Net {

DatagramSocket::DatagramSocket(Family f, int p):
	Socket(f, SOCK_DGRAM, p)
{
#ifdef WIN32
	WSAEventSelect(handle, event, FD_READ|FD_CLOSE);
#endif
	set_events(IO::P_INPUT);
}

int DatagramSocket::connect(const SockAddr &addr)
{
	check_state(false);

	sockaddr_storage sa;
	socklen_t size = addr.fill_sockaddr(sa);

	int err = ::connect(handle, reinterpret_cast<sockaddr *>(&sa), size);
	if(err==-1)
		throw SystemError("Unable to connect", errno);

	delete peer_addr;
	peer_addr = addr.copy();

	delete local_addr;
	size = sizeof(sockaddr_storage);
	getsockname(handle, reinterpret_cast<sockaddr *>(&sa), &size);
	local_addr = SockAddr::create(sa);

	connected = true;

	return (err==0)?0:1;
}

unsigned DatagramSocket::sendto(const char *buf, unsigned size, const SockAddr &addr_)
{
	check_state(false);

	if(size==0)
		return 0;

	sockaddr_storage addr;
	socklen_t addr_len = addr_.fill_sockaddr(addr);

	int ret = ::sendto(handle, buf, size, 0, reinterpret_cast<sockaddr *>(&addr), addr_len);
	if(ret<0)
	{
		if(errno==EAGAIN)
			return 0;
		else
			throw SystemError("Sendto failed", errno);
	}

	return ret;
}

unsigned DatagramSocket::recvfrom(char *buf, unsigned size, SockAddr *&addr_)
{
	check_state(false);

	if(size==0)
		return 0;

	sockaddr_storage addr;
	socklen_t addr_len = sizeof(sockaddr_storage);

	int ret = ::recvfrom(handle, buf, size, 0, reinterpret_cast<sockaddr *>(&addr), &addr_len);
	if(ret<0)
	{
		if(errno==EAGAIN)
			return 0;
		else
			throw SystemError("Recvfrom failed", errno);
	}

	addr_ = SockAddr::create(addr);

	return ret;
}

} // namespace Net
} // namespace Msp
